--[[It'll be simple. 
When totem is built we find trees near it and replace them with transparent copies.
When totem is removed in any way, we search for transparent trees and replace them with ordinary.
The only complication to those rules will be the creation of correspondence between tree types. 
And hypotetical interaction with other mods.
]]
require "defines"
radius=32--distance at which totem will have effect

local entity_transform = script.generate_event_name();
remote.add_interface("TransparentTrees",
{
--pass name of entity_transform event to other mods
entity_transform_event=function() return entity_transform end,
--debug functions
init=function() init() end,
dump=function()
    print("trees")
    print(serpent.block(global.trees))
    print("ghosts")
    print(serpent.block(global.ghosts))
    end,
})


function init()--this one makes correspondence table 
    global.trees={} --this will link trees to transparent trees
    global.ghosts={} --this will link transparent trees to trees
    for _,ent in pairs(game.entity_prototypes) do--loop over entity prototypes
        if ent.type=='tree' then--filter relevant prototypes
            if string.find(ent.name,"trnsp%-#") then
            --will only make entries for which there are pairs of transparent and untransparent trees
                _,_,name=string.find(ent.name,"trnsp%-#(.+)#")
                global.trees[name]=ent.name
                global.ghosts[ent.name]=name
            end
        end
    end
end

--we'll call that table building function when first starting the mod
script.on_init(init)
--and if there's been change in mods, which might mean new trees are added or removed
script.on_configuration_changed(init)

--transformation of trees to transparent trees
script.on_event({defines.events.on_built_entity,defines.events.on_robot_built_entity},
    function(event)
        if event.created_entity.name=='tree-transparenter' then
            pole_check(event.created_entity,global.trees)
        end
    end)
--transformation of ghostly trees to normal
script.on_event({defines.events.on_entity_died,defines.events.on_robot_pre_mined_item,defines.events.on_preplayer_mined_item,},
    function(event)
        if event.entity.name=='tree-transparenter' then
            pole_check(event.entity,global.ghosts)
        end
    end)

function pole_check(pole,list)
    --function which handles transition of trees in and out of ghostly state
    --pole is the entity of pole that is built or destroyed,
    --list is either global.trees for "in" transition or global.ghosts for "out" transition
    local p=pole.position
    local s=pole.surface
    --find and process all trees in range
    for _,tree in pairs(s.find_entities_filtered{area={{p.x-radius,p.y-radius},{p.x+radius,p.y+radius}},type="tree"}) do
        tree_swap(tree,list)
    end
end

function tree_swap(tree,list)
--takes a tree and a table according to which tree should be transformed
--along with transformation raises event for other mods to learn about it
    if list[tree.name] then--if it's listed in the table of handled trees then work with it
        new_tree=tree.surface.create_entity{name=list[tree.name],force=tree.force,position=tree.position,orientation=tree.orientation}--spawn transparent tree
        --raise event to let other mods know of change
        game.raise_event(entity_transform,{old=tree,new=new_tree})--useless unless other mod was written with anticipation of such event
        if tree.valid then tree.destroy() end--if tree didn't get destroyed durin the event, now it's time
    end
end


--A handler for this mod to account for other mods changing trees
--checks whether old entity was ghostly tree and new entity is tree too
--and tries to replace new entity with new ghostly tree
--probably needs more thought
script.on_event(entity_transform,
function(event)
    if (event.new.type=='tree') and (global.ghosts[event.old.name]) then
        if event.new.name~=global.ghosts[event.old.name] then
            tree_swap(event.new,global.trees)
        end
    end
end)
